@@ -1,3 +1,8 @@
+from __future__ import unicode_literals
+
+import six
+
+
 class APIError(Exception):
     def __init__(self, http_status, error_code, rsp=None, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
@@ -6,7 +11,7 @@ class APIError(Exception):
         self.rsp = rsp
 
     def __str__(self):
-        code_str = "HTTP %d" % self.http_status
+        code_str = 'HTTP %d' % self.http_status
 
         if self.error_code:
             code_str += ', API Error %d' % self.error_code
@@ -28,12 +33,16 @@ class BadRequestError(APIError):
         if self.rsp and 'fields' in self.rsp:
             lines.append('')
 
-            for field, error in self.rsp['fields'].iteritems():
+            for field, error in six.iteritems(self.rsp['fields']):
                 lines.append('    %s: %s' % (field, '; '.join(error)))
 
         return '\n'.join(lines)
 
 
+class CacheError(Exception):
+    """An exception for caching errors."""
+
+
 class ServerInterfaceError(Exception):
     def __init__(self, msg, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
