@@ -6,7 +6,8 @@ import re
 import six
 import sys
 
-from rbtools.utils.process import die, execute
+from rbtools.clients.errors import SCMError
+from rbtools.utils.process import execute
 
 
 # The clients are lazy loaded via load_scmclients()
@@ -139,7 +140,7 @@ class SCMClient(object):
         elif 'TREES' in config:
             trees = config['TREES']
             if not isinstance(trees, dict):
-                die('Warning: "TREES" in config file is not a dict!')
+                raise ValueError('"TREES" in config file is not a dict!')
 
             # If repository_info is a list, check if any one entry is in trees.
             path = None
@@ -224,7 +225,8 @@ class SCMClient(object):
 
         if (patch_output and patch_output.startswith('patch: **** ') and
             patch_output != only_garbage_in_patch):
-            die('Failed to execute command: %s\n%s' % (cmd, patch_output))
+            raise SCMError('Failed to execute command: %s\n%s'
+                           % (cmd, patch_output))
 
         # Check the patch for any added/deleted empty files to handle.
         if self.supports_empty_files():
@@ -242,7 +244,8 @@ class SCMClient(object):
             # is probably malformed.
             if (patch_output == only_garbage_in_patch and
                 not patched_empty_files):
-                die('Failed to execute command: %s\n%s' % (cmd, patch_output))
+                raise SCMError('Failed to execute command: %s\n%s'
+                               % (cmd, patch_output))
 
         # TODO: Should this take into account apply_patch_for_empty_files ?
         #       The return value of that function is False both when it fails
@@ -391,7 +394,9 @@ def load_scmclients(config, options):
 
     for ep in pkg_resources.iter_entry_points(group='rbtools_scm_clients'):
         try:
-            SCMCLIENTS[ep.name] = ep.load()(config=config, options=options)
+            client = ep.load()(config=config, options=options)
+            client.entrypoint_name = ep.name
+            SCMCLIENTS[ep.name] = client
         except Exception:
             logging.exception('Could not load SCM Client "%s"', ep.name)
 
