@@ -5,14 +5,16 @@ import os
 import six
 import sys
 import threading
+from collections import deque
+
 from pkg_resources import parse_version
 
 from rbtools.api.errors import APIError
 from rbtools.clients import SCMClient, RepositoryInfo
-from rbtools.clients.errors import InvalidRevisionSpecError
+from rbtools.clients.errors import InvalidRevisionSpecError, SCMError
 from rbtools.utils.checks import check_gnu_diff, check_install
 from rbtools.utils.filesystem import make_tempfile
-from rbtools.utils.process import die, execute
+from rbtools.utils.process import execute
 
 # This specific import is necessary to handle the paths for
 # cygwin enabled machines.
@@ -120,8 +122,9 @@ class ClearCaseClient(SCMClient):
                 # because webview types also list the 'snapshot'
                 # entry in properties.
                 if 'webview' in properties:
-                    die("Webviews are not supported. You can use rbt commands"
-                        " only in dynamic or snapshot views.")
+                    raise SCMError('Webviews are not supported. You can use '
+                                   'rbt commands only in dynamic or snapshot '
+                                   'views.')
                 if 'dynamic' in properties:
                     self.viewtype = 'dynamic'
                 else:
@@ -133,18 +136,19 @@ class ClearCaseClient(SCMClient):
         vobstag = execute(["cleartool", "describe", "-short", "vob:."],
                           ignore_errors=True).strip()
         if "Error: " in vobstag:
-            die("To generate diff run rbt inside vob.")
+            raise SCMError("Failed to generate diff run rbt inside vob.")
 
         root_path = execute(["cleartool", "pwv", "-root"],
                             ignore_errors=True).strip()
         if "Error: " in root_path:
-            die("To generate diff run rbt inside view.")
+            raise SCMError("Failed to generate diff run rbt inside view.")
 
-        # From current working directory cut path to VOB.
-        # VOB's tag contain backslash character before VOB's name.
-        # I hope that first character of VOB's tag like '\new_proj'
-        # won't be treat as new line character but two separate:
-        # backslash and letter 'n'
+        # From current working directory cut path to VOB. On Windows
+        # and under cygwin, the VOB tag contains the VOB's path including
+        # name, e.g. `\new_proj` for a VOB `new_proj` mounted at the root
+        # of a drive. On Unix, the VOB tag is similar, but with a different
+        # path separator, e.g. `/vobs/new_proj` for our new_proj VOB mounted
+        # at `/vobs`.
         cwd = os.getcwd()
         base_path = cwd[:len(root_path) + len(vobstag)]
 
@@ -337,8 +341,8 @@ class ClearCaseClient(SCMClient):
             #                             and the working directory
         elif n_revs == 2:
             if self.viewtype != 'dynamic':
-                die('To generate a diff using multiple revisions, you must '
-                    'use a dynamic view.')
+                raise SCMError('To generate a diff using multiple revisions, '
+                               'you must use a dynamic view.')
 
             if (revisions[0].startswith(self.REVISION_LABEL_PREFIX) and
                 revisions[1].startswith(self.REVISION_LABEL_PREFIX)):
@@ -409,7 +413,8 @@ class ClearCaseClient(SCMClient):
                 }
 
             if version_number == 0:
-                die("Unexepected version_number=0 in activity changeset")
+                raise SCMError('Unexepected version_number=0 in activity '
+                               'changeset')
             elif version_number > changelist[path]['highest']:
                 changelist[path]['highest'] = version_number
                 changelist[path]['current'] = current
@@ -728,7 +733,7 @@ class ClearCaseClient(SCMClient):
                 if self._is_a_label(lb):
                     self._remove_label(lb)
             if error_message:
-                die('Label comparison failed because:\n%s' % error_message)
+                raise SCMError('Label comparison failed:\n%s' % error_message)
 
         return changeset
 
@@ -909,7 +914,7 @@ class ClearCaseRepositoryInfo(RepositoryInfo):
         The point of this function is to find a repository on the server that
         matches self, even if the paths aren't the same. (For example, if self
         uses an 'http' path, but the server uses a 'file' path for the same
-        repository.) It does this by comparing VOB's name and uuid. If the
+        repository.) It does this by comparing the VOB's name and uuid. If the
         repositories use the same path, you'll get back self, otherwise you'll
         get a different ClearCaseRepositoryInfo object (with a different path).
         """
@@ -918,32 +923,32 @@ class ClearCaseRepositoryInfo(RepositoryInfo):
         uuid = self._get_vobs_uuid(self.vobstag)
         logging.debug("Repository's %s uuid is %r" % (self.vobstag, uuid))
 
-        repositories = server.get_repositories()
-
-        # To reduce HTTP requests (_get_repository_info call), we build an
+        # To reduce HTTP requests (_get_repository_info calls), we build an
         # ordered list of ClearCase repositories starting with the ones that
-        # have a matching vobstag.
-        repository_scan_order = []
+        # have a similar vobstag.
+        repository_scan_order = deque()
+
+        # Because the VOB tag is platform-specific, we split and search
+        # for the remote name in any sub-part so this HTTP request
+        # optimization can work for users on both Windows and Unix-like
+        # platforms.
+        vob_tag_parts = self.vobstag.split(cpath.sep)
 
         # Reduce list of repositories to only ClearCase ones and sort them by
-        # repo name matching vobstag first.
-        try:
-            while True:
-                for repository in repositories:
-                    # Ignore non-ClearCase repositories
-                    if repository['tool'] != 'ClearCase':
-                        continue
+        # repo name matching vobstag (or some part of the vobstag) first.
+        for repository in server.get_repositories(tool='ClearCase').all_items:
+            # Ignore non-ClearCase repositories.
+            if repository['tool'] != 'ClearCase':
+                continue
 
-                    # Add repos where the vobstag matches at the beginning and
-                    # others at the end.
-                    if repository['name'] == self.vobstag:
-                        repository_scan_order.insert(0, repository)
-                    else:
-                        repository_scan_order.append(repository)
+            repo_name = repository['name']
 
-                repositories = repositories.get_next()
-        except StopIteration:
-            pass
+            # Repositories with a similar VOB tag get put at the beginning and
+            # the others at the end.
+            if repo_name == self.vobstag or repo_name in vob_tag_parts:
+                repository_scan_order.appendleft(repository)
+            else:
+                repository_scan_order.append(repository)
 
         # Now try to find a matching uuid
         for repository in repository_scan_order:
@@ -958,8 +963,8 @@ class ClearCaseRepositoryInfo(RepositoryInfo):
                     # We can safely ignore this repository unless the VOB tag
                     # matches.
                     if repo_name == self.vobstag:
-                        die('You do not have permission to access this '
-                            'repository.')
+                        raise SCMError('You do not have permission to access '
+                                       'this repository.')
 
                     continue
                 else:
