@@ -21,7 +21,7 @@ from rbtools.clients.errors import (AmendError,
                                     TooManyRevisionsError)
 from rbtools.utils.checks import check_gnu_diff, check_install
 from rbtools.utils.filesystem import make_empty_files, make_tempfile
-from rbtools.utils.process import die, execute
+from rbtools.utils.process import execute
 
 
 class P4Wrapper(object):
@@ -259,7 +259,7 @@ class PerforceClient(SCMClient):
                 hostname, port = parts
 
             if not hostname:
-                die('Path %s is not a valid Perforce P4PORT' % repository_path)
+                raise SCMError('Path %s is not a valid Perforce P4PORT' % repository_path)
 
             info = socket.gethostbyaddr(hostname)
 
@@ -580,8 +580,8 @@ class PerforceClient(SCMClient):
             try:
                 changetype_short = action_mapping[action]
             except KeyError:
-                die('Unsupported action type "%s" for %s'
-                    % (action, depot_file))
+                raise SCMError('Unsupported action type "%s" for %s'
+                               % (action, depot_file))
 
             if changetype_short == 'M':
                 try:
@@ -589,7 +589,9 @@ class PerforceClient(SCMClient):
                         depot_file, local_file, base_revision, tip,
                         cl_is_shelved, False)
                 except ValueError as e:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
             elif changetype_short == 'A':
                 # Perforce has a charming quirk where the revision listed for
@@ -609,20 +611,26 @@ class PerforceClient(SCMClient):
                         depot_file, local_file, tip, cl_is_shelved,
                         cl_is_pending)
                 except ValueError as e:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
 
                 if os.path.islink(new_file):
-                    logging.warning('Skipping symlink %s', new_file)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping symlink %s', new_file)
+
                     continue
             elif changetype_short == 'D':
                 try:
                     old_file, new_file = self._extract_delete_files(
                         depot_file, base_revision)
                 except ValueError as e:
-                    logging.warning(
-                        'Skipping file %s#%s: %s',
-                        depot_file, base_revision, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning(
+                                'Skipping file %s#%s: %s',
+                                depot_file, base_revision, e)
+
                     continue
             elif changetype_short == 'MV-a':
                 # The server supports move information. We ignore this
@@ -635,7 +643,9 @@ class PerforceClient(SCMClient):
                         self._extract_move_files(
                             depot_file, tip, base_revision, cl_is_shelved)
                 except ValueError as e:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
 
             dl = self._do_diff(old_file, new_file, depot_file, base_revision,
@@ -711,9 +721,11 @@ class PerforceClient(SCMClient):
                 try:
                     cln = int(file_entry[change_key])
                 except ValueError:
-                    logging.warning('Skipping file %s: unable to parse '
-                                    'change number "%s"',
-                                    depot_file, file_entry[change_key])
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: unable to parse '
+                                        'change number "%s"',
+                                        depot_file, file_entry[change_key])
+
                     break
 
                 if action == 'integrate':
@@ -730,9 +742,11 @@ class PerforceClient(SCMClient):
                     rev_key = 'rev%d' % cid
                     rev = int(file_entry[rev_key])
                 except ValueError:
-                    logging.warning('Skipping file %s: unable to parse '
-                                    'revision number "%s"',
-                                    depot_file, file_entry[rev_key])
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: unable to parse '
+                                        'revision number "%s"',
+                                        depot_file, file_entry[rev_key])
+
                     break
 
                 change = {
@@ -793,8 +807,10 @@ class PerforceClient(SCMClient):
             try:
                 local_file = self._depot_to_local(depot_file)
             except SCMError:
-                logging.warning('Could not find local filename for "%s"',
-                                depot_file)
+                if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                    logging.warning('Could not find local filename for "%s"',
+                                    depot_file)
+
                 local_file = None
 
             rev = f['rev']
@@ -814,7 +830,9 @@ class PerforceClient(SCMClient):
                     old_file, new_file = self._extract_add_files(
                         depot_file, local_file, rev, False, False)
                 except ValueError as e:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
 
                 diff_lines += self._do_diff(
@@ -825,7 +843,9 @@ class PerforceClient(SCMClient):
                     old_file, new_file = self._extract_delete_files(
                         initial_depot_file, initial_rev)
                 except ValueError:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
 
                 diff_lines += self._do_diff(
@@ -836,7 +856,9 @@ class PerforceClient(SCMClient):
                     old_file, new_file = self._extract_edit_files(
                         depot_file, local_file, initial_rev, rev, False, True)
                 except ValueError:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
 
                 diff_lines += self._do_diff(
@@ -849,7 +871,9 @@ class PerforceClient(SCMClient):
                     old_file_b, new_file_b = self._extract_delete_files(
                         initial_depot_file, initial_rev)
                 except ValueError:
-                    logging.warning('Skipping file %s: %s', depot_file, e)
+                    if not self.config.get('SUPPRESS_CLIENT_WARNINGS', False):
+                        logging.warning('Skipping file %s: %s', depot_file, e)
+
                     continue
 
                 if supports_moves:
@@ -1063,7 +1087,8 @@ class PerforceClient(SCMClient):
             m = r_revision_range.match(path)
 
             if not m:
-                die('Path %r does not match a valid Perforce path.' % (path,))
+                raise SCMError('Path %s does not match a valid Perforce path.'
+                               % path)
             revision1 = m.group('revision1')
             revision2 = m.group('revision2')
             first_rev_path = m.group('path')
@@ -1239,7 +1264,7 @@ class PerforceClient(SCMClient):
                 # Thu Sep  3 11:24:48 2007
                 m = self.DATE_RE.search(dl[1])
                 if not m:
-                    die('Unable to parse diff header: %s' % dl[1])
+                    raise SCMError('Unable to parse diff header: %s' % dl[1])
 
                 month_map = {
                     b'Jan': b'01',
@@ -1278,7 +1303,7 @@ class PerforceClient(SCMClient):
             if not dl[-1].endswith(b'\n'):
                 dl.append(b'\n')
         else:
-            die('ERROR, no valid diffs: %s' % dl[0].decode('utf-8'))
+            raise SCMError('No valid diffs: %s' % dl[0].decode('utf-8'))
 
         return dl
 
