@@ -1,3 +1,5 @@
+from __future__ import unicode_literals
+
 from rbtools.clients import print_clients
 from rbtools.commands import Command
 
@@ -9,4 +11,4 @@ class ListRepoTypes(Command):
     description = 'Print a list of supported repository types.'
 
     def main(self, *args):
-        print_clients(self.options)
+        print_clients(self.config, self.options)
