@@ -11,10 +11,12 @@ from rbtools.utils.commands import (AlreadyStampedError,
                                     get_review_request,
                                     stamp_commit_with_review_url)
 from rbtools.utils.console import confirm
+from rbtools.utils.process import execute
 from rbtools.utils.review_request import (get_draft_or_current_value,
                                           get_revisions,
                                           guess_existing_review_request)
 
+
 class Post(Command):
     """Create and update review requests."""
     name = 'post'
@@ -22,6 +24,9 @@ class Post(Command):
     description = 'Uploads diffs to create and update review requests.'
     args = '[revisions]'
 
+    #: Reserved built-in fields that can be set using the ``--field`` argument.
+    reserved_fields = ('description', 'testing-done', 'summary')
+
     GUESS_AUTO = 'auto'
     GUESS_YES = 'yes'
     GUESS_NO = 'no'
@@ -99,9 +104,10 @@ class Post(Command):
                        dest='diff_only',
                        action='store_true',
                        default=False,
-                       help='Uploads a new diff, but does not update '
-                            'fields from the change description '
-                            '(Perforce/Plastic only).'),
+                       help='Uploads a new diff, but does not automatically '
+                            'update fields from the commit message/change '
+                            'description. Fields explicitly provided by '
+                            'other options will be ignored.'),
             ]
         ),
         Command.server_options,
@@ -111,6 +117,15 @@ class Post(Command):
             description='Options for setting the contents of fields in the '
                         'review request.',
             option_list=[
+                Option('-f', '--field',
+                       dest='fields',
+                       action='append',
+                       default=None,
+                       metavar='FIELD_NAME=VALUE',
+                       help='Sets custom fields into the extra_data of a '
+                            'review request. Can also be used to set '
+                            'built-in fields like description, summary, '
+                            'testing-done.'),
                 Option('-g', '--guess-fields',
                        dest='guess_fields',
                        action='store',
@@ -131,7 +146,7 @@ class Post(Command):
                        action='store',
                        config_key='GUESS_SUMMARY',
                        nargs='?',
-                       default=GUESS_AUTO,
+                       default=None,
                        const=GUESS_YES,
                        choices=GUESS_CHOICES,
                        help='Generates the Summary field based on the '
@@ -146,7 +161,7 @@ class Post(Command):
                        action='store',
                        config_key='GUESS_DESCRIPTION',
                        nargs='?',
-                       default=GUESS_AUTO,
+                       default=None,
                        const=GUESS_YES,
                        choices=GUESS_CHOICES,
                        help='Generates the Description field based on the '
@@ -237,6 +252,7 @@ class Post(Command):
             ]
         ),
         Command.diff_options,
+        Command.branch_options,
         Command.perforce_options,
         Command.subversion_options,
         Command.tfs_options,
@@ -245,13 +261,49 @@ class Post(Command):
     def post_process_options(self):
         super(Post, self).post_process_options()
 
-        # -g implies --guess-summary and --guess-description
-        if self.options.guess_fields:
-            self.options.guess_fields = self.normalize_guess_value(
-                self.options.guess_fields, '--guess-fields')
+        extra_fields = {}
 
-            self.options.guess_summary = self.options.guess_fields
-            self.options.guess_description = self.options.guess_fields
+        if self.options.fields is None:
+            self.options.fields = []
+
+        for field in self.options.fields:
+            key_value_pair = field.split('=', 1)
+
+            if len(key_value_pair) != 2:
+                raise CommandError(
+                    'The --field argument should be in the form of: '
+                    '--field name=value; got "%s" instead.'
+                    % field
+                )
+
+            key, value = key_value_pair
+
+            if key in self.reserved_fields:
+                key_var = key.replace('-', '_')
+
+                if getattr(self.options, key_var):
+                    raise CommandError(
+                        'The "{0}" field was provided by both --{0}= '
+                        'and --field {0}=. Please use --{0} instead.'
+                        .format(key)
+                    )
+
+                setattr(self.options, key_var, value)
+            else:
+                extra_fields['extra_data.%s' % key] = value
+
+        self.options.extra_fields = extra_fields
+
+        # -g implies --guess-summary and --guess-description
+        self.options.guess_fields = self.normalize_guess_value(
+            self.options.guess_fields, '--guess-fields')
+
+        for field_name in ('guess_summary', 'guess_description'):
+            # We want to ensure we only override --guess-{field} with
+            # --guess-fields when --guess-{field} is not provided.
+            # to the default (auto).
+            if getattr(self.options, field_name) is None:
+                setattr(self.options, field_name, self.options.guess_fields)
 
         if self.options.revision_range:
             raise CommandError(
@@ -340,19 +392,16 @@ class Post(Command):
         with those one the server, and return the first match.
         """
         if isinstance(repository_info.path, list):
-            repositories = api_root.get_repositories(only_fields='path',
-                                                     only_links='')
+            repositories = api_root.get_repositories(
+                only_fields='path,mirror_path', only_links='')
 
-            try:
-                while True:
-                    for repo in repositories:
-                        if repo['path'] in repository_info.path:
-                            repository_info.path = repo['path']
-                            raise StopIteration()
-
-                    repositories = repositories.get_next()
-            except StopIteration:
-                pass
+            for repo in repositories.all_items:
+                if repo['path'] in repository_info.path:
+                    repository_info.path = repo['path']
+                    break
+                elif repo['mirror_path'] in repository_info.path:
+                    repository_info.path = repo['mirror_path']
+                    break
 
         if isinstance(repository_info.path, list):
             error_str = [
@@ -428,10 +477,6 @@ class Post(Command):
                         review_request_id=rid,
                         only_fields='absolute_url,bugs_closed,id,status',
                         only_links='diffs,draft')
-
-                    if not self.options.diff_only:
-                        review_request = review_request.update(
-                            changenum=changenum)
                 else:
                     raise CommandError('Error creating review request: %s' % e)
 
@@ -508,62 +553,66 @@ class Post(Command):
                     print('Could not stamp review URL onto the commit '
                           'message.')
 
-        # If the user has requested to guess the summary or description,
-        # get the commit message and override the summary and description
-        # options. The guessing takes place after stamping so that the
-        # guessed description matches the commit when rbt exits.
-        if not self.options.diff_filename:
-            self.check_guess_fields()
-
         # Update the review request draft fields based on options set
         # by the user, or configuration.
         update_fields = {}
 
-        if self.options.target_groups:
-            update_fields['target_groups'] = self.options.target_groups
-
-        if self.options.target_people:
-            update_fields['target_people'] = self.options.target_people
-
-        if self.options.depends_on:
-            update_fields['depends_on'] = self.options.depends_on
-
-        if self.options.summary:
-            update_fields['summary'] = self.options.summary
-
-        if self.options.branch:
-            update_fields['branch'] = self.options.branch
-
-        if self.options.bugs_closed:
-            # Append to the existing list of bugs.
-            self.options.bugs_closed = self.options.bugs_closed.strip(', ')
-            bug_set = (set(re.split('[, ]+', self.options.bugs_closed)) |
-                       set(review_request.bugs_closed))
-            self.options.bugs_closed = ','.join(bug_set)
-            update_fields['bugs_closed'] = self.options.bugs_closed
-
-        if self.options.description:
-            update_fields['description'] = self.options.description
-
-        if self.options.testing_done:
-            update_fields['testing_done'] = self.options.testing_done
-
-        if ((self.options.description or self.options.testing_done) and
-            self.options.markdown and
-            self.tool.capabilities.has_capability('text', 'markdown')):
-            # The user specified that their Description/Testing Done are
-            # valid Markdown, so tell the server so it won't escape the text.
-            update_fields['text_type'] = 'markdown'
-
-        if self.options.change_description:
-            update_fields['changedescription'] = \
-                self.options.change_description
-
         if self.options.publish:
             update_fields['public'] = True
 
-        if supports_posting_commit_ids and commit_id != draft.commit_id:
-            update_fields['commit_id'] = commit_id or ''
+        if not self.options.diff_only:
+            # If the user has requested to guess the summary or description,
+            # get the commit message and override the summary and description
+            # options, which we'll fill in below. The guessing takes place
+            # after stamping so that the guessed description matches the commit
+            # when rbt exits.
+            if not self.options.diff_filename:
+                self.check_guess_fields()
+
+            update_fields.update(self.options.extra_fields)
+
+            if self.options.target_groups:
+                update_fields['target_groups'] = self.options.target_groups
+
+            if self.options.target_people:
+                update_fields['target_people'] = self.options.target_people
+
+            if self.options.depends_on:
+                update_fields['depends_on'] = self.options.depends_on
+
+            if self.options.summary:
+                update_fields['summary'] = self.options.summary
+
+            if self.options.branch:
+                update_fields['branch'] = self.options.branch
+
+            if self.options.bugs_closed:
+                # Append to the existing list of bugs.
+                self.options.bugs_closed = self.options.bugs_closed.strip(', ')
+                bug_set = (set(re.split('[, ]+', self.options.bugs_closed)) |
+                           set(review_request.bugs_closed))
+                self.options.bugs_closed = ','.join(bug_set)
+                update_fields['bugs_closed'] = self.options.bugs_closed
+
+            if self.options.description:
+                update_fields['description'] = self.options.description
+
+            if self.options.testing_done:
+                update_fields['testing_done'] = self.options.testing_done
+
+            if ((self.options.description or self.options.testing_done) and
+                self.options.markdown and
+                self.tool.capabilities.has_capability('text', 'markdown')):
+                # The user specified that their Description/Testing Done are
+                # valid Markdown, so tell the server so it won't escape the text.
+                update_fields['text_type'] = 'markdown'
+
+            if self.options.change_description:
+                update_fields['changedescription'] = \
+                    self.options.change_description
+
+            if supports_posting_commit_ids and commit_id != draft.commit_id:
+                update_fields['commit_id'] = commit_id or ''
 
         if update_fields:
             try:
@@ -606,12 +655,24 @@ class Post(Command):
                 commit_message = self.tool.get_commit_message(self.revisions)
 
                 if commit_message:
-                    if guess_summary:
-                        self.options.summary = commit_message['summary']
+                    guessed_summary = commit_message['summary']
+                    guessed_description = commit_message['description']
 
-                    if guess_description:
-                        self.options.description = \
-                            commit_message['description']
+                    if guess_summary and guess_description:
+                        self.options.summary = guessed_summary
+                        self.options.description = guessed_description
+                    elif guess_summary:
+                        self.options.summary = guessed_summary
+                    elif guess_description:
+                        # If we're guessing the description but not the summary
+                        # (for example, if --summary was included), we probably
+                        # don't want to strip off the summary line of the
+                        # commit message.
+                        if guessed_description.startswith(guessed_summary):
+                            self.options.description = guessed_description
+                        else:
+                            self.options.description = \
+                                guessed_summary + '\n\n' + guessed_description
             except NotImplementedError:
                 # The SCMClient doesn't support getting commit messages,
                 # so we can't provide the guessed versions.
@@ -695,6 +756,12 @@ class Post(Command):
             base_commit_id = diff_info.get('base_commit_id')
             commit_id = diff_info.get('commit_id')
 
+            logging.debug('Generated diff size: %d bytes', len(diff))
+
+            if parent_diff:
+                logging.debug('Generated parent diff size: %d bytes',
+                              len(parent_diff))
+
         repository = (
             self.options.repository_name or
             self.options.repository_url or
@@ -764,7 +831,10 @@ class Post(Command):
         else:
             changenum = self.tool.get_changenum(self.revisions)
 
-        commit_id = changenum
+        # Not all scm clients support get_changenum, so if get_changenum
+        # returns None (the default for clients that don't have changenums),
+        # we'll prefer the existing commit_id.
+        commit_id = changenum or commit_id
 
         if self.options.update and self.revisions:
             review_request = guess_existing_review_request(
@@ -808,14 +878,25 @@ class Post(Command):
 
         # Load the review up in the browser if requested to.
         if self.options.open_browser:
-            try:
-                import webbrowser
-                if 'open_new_tab' in dir(webbrowser):
-                    # open_new_tab is only in python 2.5+
-                    webbrowser.open_new_tab(review_url)
-                elif 'open_new' in dir(webbrowser):
-                    webbrowser.open_new(review_url)
-                else:
-                    os.system('start %s' % review_url)
-            except:
-                logging.error('Error opening review URL: %s' % review_url)
+            if sys.platform == 'darwin':
+                # The 'webbrowser' module currently does a bunch of stuff with
+                # AppleScript, which is broken on macOS 10.12.5. See
+                # https://bugs.python.org/issue30392 for more discussion.
+                try:
+                    execute(['open', review_url])
+                except Exception as e:
+                    logging.exception('Error opening review URL %s: %s',
+                                      review_url, e)
+            else:
+                try:
+                    import webbrowser
+                    if 'open_new_tab' in dir(webbrowser):
+                        # open_new_tab is only in python 2.5+
+                        webbrowser.open_new_tab(review_url)
+                    elif 'open_new' in dir(webbrowser):
+                        webbrowser.open_new(review_url)
+                    else:
+                        os.system('start %s' % review_url)
+                except Exception as e:
+                    logging.exception('Error opening review URL %s: %s',
+                                      review_url, e)
