@@ -1,7 +1,11 @@
+from __future__ import print_function, unicode_literals
+
 import os
 import subprocess
 from distutils.util import strtobool
 
+from six.moves import input
+
 from rbtools.utils.filesystem import make_tempfile
 
 
@@ -13,11 +17,12 @@ def confirm(question):
     'No' values: n, no , f, false, off, 0
     """
     while True:
+        full_question = '%s [Yes/No]: ' % question
+        answer = input(full_question.encode('utf-8')).lower()
         try:
-            answer = raw_input("%s [Yes/No]: " % question).lower()
             return strtobool(answer)
         except ValueError:
-            print '%s is not a valid answer.' % answer
+            print('%s is not a valid answer.' % answer)
 
 
 def edit_text(content):
@@ -29,9 +34,10 @@ def edit_text(content):
     tempfile = make_tempfile(content.encode('utf8'))
     editor = os.environ.get('VISUAL') or os.environ.get('EDITOR') or 'vi'
     try:
-        subprocess.call([editor, tempfile])
+        subprocess.call(editor.split() + [tempfile])
     except OSError:
-        print 'No editor found. Set EDITOR environment variable or install vi.'
+        print('No editor found. Set EDITOR environment variable or install '
+              'vi.')
         raise
 
     f = open(tempfile)
